@@ -7,7 +7,7 @@ RDF::Query::Algebra::BasicGraphPattern - Algebra class for BasicGraphPattern pat
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::BasicGraphPattern version 2.903_02.
+This document describes RDF::Query::Algebra::BasicGraphPattern version 2.904_01.
 
 =cut
 
@@ -30,7 +30,7 @@ use RDF::Trine::Iterator qw(smap swatch);
 our ($VERSION);
 my %AS_SPARQL;
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -114,7 +114,7 @@ sub quads {
 
 =item C<< sse >>
 
-Returns the SSE string for this alegbra expression.
+Returns the SSE string for this algebra expression.
 
 =cut
 
@@ -133,7 +133,7 @@ sub sse {
 
 =item C<< as_sparql >>
 
-Returns the SPARQL string for this alegbra expression.
+Returns the SPARQL string for this algebra expression.
 
 =cut
 
@@ -172,6 +172,21 @@ sub as_hash {
 	};
 }
 
+=item C<< as_spin ( $model ) >>
+
+Adds statements to the given model to represent this algebra object in the
+SPARQL Inferencing Notation (L<http://www.spinrdf.org/>).
+
+=cut
+
+sub as_spin {
+	my $self	= shift;
+	my $model	= shift;
+	my @t		= $self->triples;
+	my @nodes	= map { $_->as_spin( $model ) } @t;
+	return @nodes;
+}
+
 =item C<< type >>
 
 Returns the type of this algebra expression.
@@ -237,7 +252,7 @@ sub _check_duplicate_blanks {
 
 =item C<< connected >>
 
-Returns true if the pattern is connected through shared variables, fase otherwise.
+Returns true if the pattern is connected through shared variables, false otherwise.
 
 =cut
 
