@@ -7,7 +7,7 @@ RDF::Query::Algebra::Construct - Algebra class for construct query results
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Construct version 2.903_02.
+This document describes RDF::Query::Algebra::Construct version 2.904_01.
 
 =cut
 
@@ -28,7 +28,7 @@ use RDF::Trine::Iterator qw(sgrep);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -99,7 +99,7 @@ sub triples {
 
 =item C<< sse >>
 
-Returns the SSE string for this alegbra expression.
+Returns the SSE string for this algebra expression.
 
 =cut
 
@@ -107,32 +107,45 @@ sub sse {
 	my $self	= shift;
 	my $context	= shift;
 	my $prefix	= shift || '';
-	my $indent	= $context->{indent};
+	my $indent	= $context->{indent} || '  ';
+	my $triples	= join("\n${prefix}${indent}${indent}", map { $_->sse( $context, "${prefix}${indent}${indent}" ) } @{$self->triples});
 	
 	return sprintf(
-		'(construct\n${prefix}${indent}%s)',
+		"(construct\n${prefix}${indent}(\n${prefix}${indent}${indent}%s\n${prefix}${indent})\n${prefix}${indent}%s)",
+		$triples,
 		$self->pattern->sse( $context, "${prefix}${indent}" ),
 	);
 }
 
 =item C<< as_sparql >>
 
-Returns the SPARQL string for this alegbra expression.
+Returns the SPARQL string for this algebra expression.
 
 =cut
 
 sub as_sparql {
 	my $self	= shift;
-	my $context	= shift;
+	my $context	= shift || {};
 	my $indent	= shift;
 	my $triples	= $self->triples;
 	my $bgp		= RDF::Query::Algebra::BasicGraphPattern->new( @$triples );
 	my $ggp		= RDF::Query::Algebra::GroupGraphPattern->new( $bgp );
-
+	my $force	= $context->{ force_ggp_braces } || 0;
+	
+	my ($template, $pattern);
+	{
+		$context->{ force_ggp_braces }	= $force + 1;
+		$template	= $ggp->as_sparql( $context, $indent );
+	}
+	{
+		$context->{ force_ggp_braces }	= $force + 1;
+		$pattern		= $self->pattern->as_sparql( $context, $indent );
+	}
+	
 	my $string	= sprintf(
 		"CONSTRUCT %s\n${indent}WHERE %s",
-		$ggp->as_sparql( $context, $indent ),
-		$self->pattern->as_sparql( $context, $indent ),
+		$template,
+		$pattern,
 	);
 	return $string;
 }
