@@ -7,7 +7,7 @@ RDF::Query::Algebra::GroupGraphPattern - Algebra class for GroupGraphPattern pat
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::GroupGraphPattern version 2.903_02.
+This document describes RDF::Query::Algebra::GroupGraphPattern version 2.904_01.
 
 =cut
 
@@ -32,7 +32,7 @@ use RDF::Trine::Iterator qw(sgrep smap swatch);
 our ($VERSION, $debug);
 BEGIN {
 	$debug		= 0;
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 	our %SERVICE_BLOOM_IGNORE	= ('http://dbpedia.org/sparql' => 1);	# by default, assume dbpedia doesn't implement k:bloom().
 }
 
@@ -126,7 +126,7 @@ sub quads {
 
 =item C<< sse >>
 
-Returns the SSE string for this alegbra expression.
+Returns the SSE string for this algebra expression.
 
 =cut
 
@@ -149,23 +149,35 @@ sub sse {
 
 =item C<< as_sparql >>
 
-Returns the SPARQL string for this alegbra expression.
+Returns the SPARQL string for this algebra expression.
 
 =cut
 
 sub as_sparql {
 	my $self	= shift;
-	my $context	= shift;
+	my $context	= shift || {};
 	my $indent	= shift || '';
+	my $force	= $context->{force_ggp_braces};
+	$force		= 0 unless (defined($force));
+	if ($force) {
+		$context->{force_ggp_braces}--;
+	}
 	
 	my @patterns;
-	foreach my $p ($self->patterns) {
-		push(@patterns, $p->as_sparql( $context, "$indent\t" ));
+	my @p	= $self->patterns;
+	
+	if (scalar(@p) == 0) {
+		return "{}";
+	} elsif (scalar(@p) == 1 and not($force)) {
+		return $p[0]->as_sparql($context, $indent);
+	} else {
+		foreach my $p (@p) {
+			push(@patterns, $p->as_sparql( $context, "$indent\t" ));
+		}
+		my $patterns	= join("\n${indent}\t", @patterns);
+		my $string		= sprintf("{\n${indent}\t%s\n${indent}}", $patterns);
+		return $string;
 	}
-	return "{}" unless (@patterns);
-	my $patterns	= join("\n${indent}\t", @patterns);
-	my $string		= sprintf("{\n${indent}\t%s\n${indent}}", $patterns);
-	return $string;
 }
 
 =item C<< as_hash >>
@@ -183,6 +195,20 @@ sub as_hash {
 	};
 }
 
+=item C<< as_spin ( $model ) >>
+
+Adds statements to the given model to represent this algebra object in the
+SPARQL Inferencing Notation (L<http://www.spinrdf.org/>).
+
+=cut
+
+sub as_spin {
+	my $self	= shift;
+	my $model	= shift;
+	return map { $_->as_spin($model) } $self->patterns;
+	
+}
+
 =item C<< type >>
 
 Returns the type of this algebra expression.
