@@ -7,7 +7,7 @@ RDF::Query::Algebra::Optional - Algebra class for Optional patterns
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Optional version 2.903_02.
+This document describes RDF::Query::Algebra::Optional version 2.904_01.
 
 =cut
 
@@ -26,7 +26,7 @@ use RDF::Trine::Iterator qw(smap sgrep swatch);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -89,7 +89,7 @@ sub optional {
 
 =item C<< sse >>
 
-Returns the SSE string for this alegbra expression.
+Returns the SSE string for this algebra expression.
 
 =cut
 
@@ -108,7 +108,7 @@ sub sse {
 
 =item C<< as_sparql >>
 
-Returns the SPARQL string for this alegbra expression.
+Returns the SPARQL string for this algebra expression.
 
 =cut
 
@@ -119,11 +119,32 @@ sub as_sparql {
 	my $string	= sprintf(
 		"%s\n${indent}OPTIONAL %s",
 		$self->pattern->as_sparql( $context, $indent ),
-		$self->optional->as_sparql( $context, $indent ),
+		$self->optional->as_sparql( { %$context, force_ggp_braces => 1 }, $indent ),
 	);
 	return $string;
 }
 
+=item C<< as_spin ( $model ) >>
+
+Adds statements to the given model to represent this algebra object in the
+SPARQL Inferencing Notation (L<http://www.spinrdf.org/>).
+
+=cut
+
+sub as_spin {
+	my $self	= shift;
+	my $model	= shift;
+	my $spin	= RDF::Trine::Namespace->new('http://spinrdf.org/spin#');
+	my $rdf		= RDF::Trine::Namespace->new('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
+	my @lhs		= $self->pattern->as_spin($model);
+	my @rhs		= $self->optional->as_spin($model);
+	my $opt		= RDF::Query::Node::Blank->new();
+	my $list	= $model->add_list( @rhs );
+	$model->add_statement( RDF::Trine::Statement->new($opt, $rdf->type, $spin->Optional) );
+	$model->add_statement( RDF::Trine::Statement->new($opt, $spin->elements, $list) );
+	return (@lhs, $opt);
+}
+
 =item C<< as_hash >>
 
 Returns the query as a nested set of plain data structures (no objects).
