@@ -7,7 +7,7 @@ RDF::Query::Algebra::Sort - Algebra class for sorting
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Sort version 2.903_02.
+This document describes RDF::Query::Algebra::Sort version 2.904_01.
 
 =cut
 
@@ -29,7 +29,7 @@ use Time::HiRes qw(gettimeofday tv_interval);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -98,7 +98,7 @@ sub orderby {
 
 =item C<< sse >>
 
-Returns the SSE string for this alegbra expression.
+Returns the SSE string for this algebra expression.
 
 =cut
 
@@ -124,15 +124,35 @@ sub sse {
 
 =item C<< as_sparql >>
 
-Returns the SPARQL string for this alegbra expression.
+Returns the SPARQL string for this algebra expression.
 
 =cut
 
 sub as_sparql {
 	my $self	= shift;
-	my $context	= shift;
+	my $context	= shift || {};
 	my $indent	= shift;
 	
+	if ($context->{ skip_sort }) {
+		$context->{ skip_sort }--;
+		return $self->pattern->as_sparql( $context, $indent );
+	}
+	
+	my $order_exprs	= $self->_as_sparql_order_exprs($context, $indent);
+	
+	my $string	= sprintf(
+		"%s\nORDER BY %s",
+		$self->pattern->as_sparql( $context, $indent ),
+		$order_exprs,
+	);
+	return $string;
+}
+
+sub _as_sparql_order_exprs {
+	my $self	= shift;
+	my $context	= shift;
+	my $intent	= shift;
+	
 	my @order_sparql;
 	my @orderby	= $self->orderby;
 	foreach my $o (@orderby) {
@@ -143,13 +163,7 @@ sub as_sparql {
 						: sprintf("%s(%s)", $dir, $val->as_sparql( $context ));
 		push(@order_sparql, $str);
 	}
-	
-	my $string	= sprintf(
-		"%s\nORDER BY %s",
-		$self->pattern->as_sparql( $context, $indent ),
-		join(' ', @order_sparql),
-	);
-	return $string;
+	return join(' ', @order_sparql);
 }
 
 =item C<< as_hash >>
