@@ -7,7 +7,7 @@ RDF::Query::Expression::Binary - Algebra class for binary expressions
 
 =head1 VERSION
 
-This document describes RDF::Query::Expression::Binary version 2.903_02.
+This document describes RDF::Query::Expression::Binary version 2.904_01.
 
 =cut
 
@@ -27,7 +27,7 @@ use Carp qw(carp croak confess);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -43,7 +43,7 @@ L<RDF::Query::Expression> class.
 
 =item C<< sse >>
 
-Returns the SSE string for this alegbra expression.
+Returns the SSE string for this algebra expression.
 
 =cut
 
@@ -60,7 +60,7 @@ sub sse {
 
 =item C<< as_sparql >>
 
-Returns the SPARQL string for this alegbra expression.
+Returns the SPARQL string for this algebra expression.
 
 =cut
 
@@ -99,17 +99,18 @@ sub evaluate {
 	
 	$l->debug("Binary Operator '$op': " . Dumper($lhs, $rhs));
 	
-	if ($op eq '+') {
-		if (blessed($lhs) and $lhs->isa('RDF::Query::Node::Literal') and blessed($rhs) and $rhs->isa('RDF::Query::Node::Literal')) {
-			if (not($lhs->has_datatype) and not($rhs->has_datatype)) {
-				my $value	= $lhs->literal_value . $rhs->literal_value;
-				return RDF::Query::Node::Literal->new( $value );
-			}
-		}
-	}
+### This does overloading of infix<+> on literal values to perform string concatenation
+# 	if ($op eq '+') {
+# 		if (blessed($lhs) and $lhs->isa('RDF::Query::Node::Literal') and blessed($rhs) and $rhs->isa('RDF::Query::Node::Literal')) {
+# 			if (not($lhs->has_datatype) and not($rhs->has_datatype)) {
+# 				my $value	= $lhs->literal_value . $rhs->literal_value;
+# 				return RDF::Query::Node::Literal->new( $value );
+# 			}
+# 		}
+# 	}
 	
 	if ($op =~ m#^[-+/*]$#) {
-		if (blessed($lhs) and blessed($rhs) and $lhs->is_numeric_type and $rhs->is_numeric_type) {
+		if (blessed($lhs) and blessed($rhs) and $lhs->isa('RDF::Query::Node::Literal') and $rhs->isa('RDF::Query::Node::Literal') and $lhs->is_numeric_type and $rhs->is_numeric_type) {
 			my $type	= $self->promote_type( $op, $lhs->literal_datatype, $rhs->literal_datatype );
 			my $value;
 			if ($op eq '+') {
