@@ -7,7 +7,7 @@ RDF::Query::Functions - Standard Extension Functions
 
 =head1 VERSION
 
-This document describes RDF::Query::Functions version 2.903_02.
+This document describes RDF::Query::Functions version 2.904_01.
 
 =head1 DESCRIPTION
 
@@ -16,6 +16,10 @@ C<< RDF::Query::Functions::* >>. Each of those modules
 has an C<install> method that simply adds coderefs
 to C<< %RDF::Query::functions >>.
 
+=head1 METHODS
+
+=over 4
+
 =cut
 
 package RDF::Query::Functions;
@@ -26,6 +30,7 @@ no warnings 'redefine';
 
 our $BLOOM_FILTER_LOADED;
 
+use Scalar::Util qw(refaddr);
 use Log::Log4perl;
 
 use Module::Pluggable
@@ -40,11 +45,31 @@ use Module::Pluggable
 our ($VERSION, $l);
 BEGIN {
 	$l			= Log::Log4perl->get_logger("rdf.query.functions");
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
 
+=item C<< install_function ( $uri, \&func ) >>
+
+=item C<< install_function ( \@uris, \&func ) >>
+
+Install the supplied CODE reference as the implementation for the given function URI(s).
+
+=cut
+
+sub install_function {
+	my $class	= shift;
+	while (@_) {
+		my $uris	= shift;
+		my $func	= shift;
+		$RDF::Query::preferred_function_name{ refaddr($func) }	= ref($uris) ? $uris->[0] : $uris;
+		foreach my $uri (ref($uris) ? @$uris : $uris) {
+			$RDF::Query::functions{$uri}	= $func;
+		}
+	}
+}
+
 foreach my $function_set (__PACKAGE__->function_sets) {
 	$function_set->install;
 }
@@ -53,6 +78,8 @@ foreach my $function_set (__PACKAGE__->function_sets) {
 
 __END__
 
+=back
+
 =head1 SEE ALSO
 
 L<RDF::Query::Functions::SPARQL>,
