@@ -7,7 +7,7 @@ RDF::Query::Node::Literal - RDF Node class for literals
 
 =head1 VERSION
 
-This document describes RDF::Query::Node::Literal version 2.903_02.
+This document describes RDF::Query::Node::Literal version 2.904_01.
 
 =cut
 
@@ -30,7 +30,7 @@ use Carp qw(carp croak confess);
 
 our ($VERSION, $LAZY_COMPARISONS);
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -125,6 +125,8 @@ sub _cmp {
 				$c	= -1;
 			} elsif (not($nb->has_datatype) or $nb->literal_datatype eq 'http://www.w3.org/2001/XMLSchema#string') {
 				$c	= $nodea->literal_value cmp $nodeb->literal_value;
+			} elsif ($LAZY_COMPARISONS) {
+				return $nodea->as_string cmp $nodeb->as_string;
 			} else {
 				throw RDF::Query::Error::TypeError -text => "Attempt to compare typed-literal with xsd:string.";
 			}
@@ -233,6 +235,17 @@ sub as_hash {
 	return $hash;
 }
 
+=item C<< is_simple_literal >>
+
+Returns true if the literal is "simple" -- is a literal without datatype or language.
+
+=cut
+
+sub is_simple_literal {
+	my $self	= shift;
+	return not($self->has_language or $self->has_datatype);
+}
+
 =item C<< is_numeric_type >>
 
 Returns true if the literal is a known (xsd) numeric type.
@@ -279,6 +292,19 @@ sub numeric_value {
 	}
 }
 
+=item C<< type_list >>
+
+Returns a two-item list suitable for use as the second and third arguments to
+RDF::Query::Node::Literal constructor. The two returned values correspond to
+literal language tag and literal datatype URI, respectively.
+
+=cut
+
+sub type_list {
+	my $self	= shift;
+	return ($self->literal_value_language, $self->literal_datatype);
+}
+
 sub DESTROY {
 	my $self	= shift;
 	my $addr	= refaddr($self);
