@@ -7,7 +7,7 @@ RDF::Query::Plan::Clear - Executable query plan for CLEAR operations.
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan::Clear version 2.903_02.
+This document describes RDF::Query::Plan::Clear version 2.904_01.
 
 =head1 METHODS
 
@@ -36,7 +36,7 @@ use RDF::Query::VariableBindings;
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.903_02';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
@@ -74,16 +74,20 @@ sub execute ($) {
 # 	warn "clearing graph " . $graph->as_string;
 	my $ok	= 0;
 	try {
-		my $uri	= $graph->uri_value;
-		if ($uri eq 'tag:gwilliams@cpan.org,2010-01-01:RT:ALL') {
-			$context->model->remove_statements( undef, undef, undef, undef );
-		} elsif ($uri eq 'tag:gwilliams@cpan.org,2010-01-01:RT:NAMED') {
-			my $citer	= $context->model->get_contexts;
-			while (my $graph = $citer->next) {
+		if ($graph->is_nil) {
+			$context->model->remove_statements( undef, undef, undef, $graph );
+		} else {
+			my $uri	= $graph->uri_value;
+			if ($uri eq 'tag:gwilliams@cpan.org,2010-01-01:RT:ALL') {
+				$context->model->remove_statements( undef, undef, undef, undef );
+			} elsif ($uri eq 'tag:gwilliams@cpan.org,2010-01-01:RT:NAMED') {
+				my $citer	= $context->model->get_contexts;
+				while (my $graph = $citer->next) {
+					$context->model->remove_statements( undef, undef, undef, $graph );
+				}
+			} else {
 				$context->model->remove_statements( undef, undef, undef, $graph );
 			}
-		} else {
-			$context->model->remove_statements( undef, undef, undef, $graph );
 		}
 		$ok		= 1;
 	} catch RDF::Trine::Error with {};
