@@ -7,10 +7,13 @@ RDF::Query::Plan::ComputedStatement - Executable query plan for computed triples
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan::ComputedStatement version 2.201, released 30 January 2010.
+This document describes RDF::Query::Plan::ComputedStatement version 2.904_01.
 
 =head1 METHODS
 
+Beyond the methods documented below, this class inherits methods from the
+L<RDF::Query::Plan> class.
+
 =over 4
 
 =cut
@@ -32,7 +35,7 @@ use RDF::Query::VariableBindings;
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.201';
+	$VERSION	= '2.904_01';
 }
 
 ######################################################################
